@@ -25,6 +25,7 @@ along with sysdig.  If not, see <http://www.gnu.org/licenses/>.
 #include <sys/resource.h>
 #include <sys/syscall.h>
 #include <sys/ioctl.h>
+#include <sys/stat.h>
 #endif
 
 #include "scap.h"
@@ -56,12 +57,15 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	int64_t tmp;
 	uint32_t uid;
 	uint64_t ppid;
+	uint64_t vpid;
+	uint64_t vtid;
 	int64_t sid;
 	uint32_t vmsize_kb;
 	uint32_t vmrss_kb;
 	uint32_t vmswap_kb;
 	uint64_t pfmajor;
 	uint64_t pfminor;
+	int32_t tty;
 	char line[512];
 	char tmpc;
 	char* s;
@@ -75,6 +79,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	tinfo->pfmajor = 0;
 	tinfo->pfminor = 0;
 	tinfo->filtered_out = 0;
+	tinfo->tty = 0;
 
 	snprintf(filename, sizeof(filename), "%sstatus", procdirname);
 
@@ -165,14 +170,38 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 				ASSERT(false);
 			}
 		}
+		else if(strstr(line, "NSpid:") == line)
+		{
+			nfound++;
+			if(sscanf(line, "NSpid: %*u %" PRIu64, &vtid) == 1)
+			{
+				tinfo->vtid = vtid;
+			}
+			else
+			{
+				tinfo->vtid = tinfo->tid;
+			}
+		}
+		else if(strstr(line, "NStgid:") == line)
+		{
+			nfound++;
+			if(sscanf(line, "NStgid: %*u %" PRIu64, &vpid) == 1)
+			{
+				tinfo->vpid = vpid;
+			}
+			else
+			{
+				tinfo->vpid = tinfo->pid;
+			}
+		}
 
-		if(nfound == 6)
+		if(nfound == 8)
 		{
 			break;
 		}
 	}
 
-	ASSERT(nfound == 6 || nfound == 5);
+	ASSERT(nfound == 8 || nfound == 6 || nfound == 5);
 
 	fclose(f);
 
@@ -203,12 +232,12 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	//
 	// Extract the line content
 	//
-	if(sscanf(s + 2, "%c %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64,
+	if(sscanf(s + 2, "%c %" PRId64 " %" PRId64 " %" PRId64 " %" PRId32 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64 " %" PRId64,
 		&tmpc,
 		&tmp,
 		&sid,
 		&tmp,
-		&tmp,
+		&tty,
 		&tmp,
 		&tmp,
 		&pfminor,
@@ -223,6 +252,7 @@ int32_t scap_proc_fill_info_from_stats(char* procdirname, struct scap_threadinfo
 	tinfo->pfmajor = pfmajor;
 	tinfo->pfminor = pfminor;
 	tinfo->sid = (uint64_t) sid;
+	tinfo->tty = tty;
 
 	fclose(f);
 	return SCAP_SUCCESS;
@@ -338,9 +368,8 @@ int32_t scap_proc_fill_cgroups(struct scap_threadinfo* tinfo, const char* procdi
 
 static int32_t scap_get_vtid(scap_t* handle, int64_t tid, int64_t *vtid)
 {
-	if(handle->m_file)
+	if(handle->m_mode != SCAP_MODE_LIVE)
 	{
-		ASSERT(false);
 		return SCAP_FAILURE;
 	}
 
@@ -363,9 +392,8 @@ static int32_t scap_get_vtid(scap_t* handle, int64_t tid, int64_t *vtid)
 
 static int32_t scap_get_vpid(scap_t* handle, int64_t tid, int64_t *vpid)
 {
-	if(handle->m_file)
+	if(handle->m_mode != SCAP_MODE_LIVE)
 	{
-		ASSERT(false);
 		return SCAP_FAILURE;
 	}
 
@@ -388,7 +416,7 @@ static int32_t scap_get_vpid(scap_t* handle, int64_t tid, int64_t *vpid)
 
 int32_t scap_getpid_global(scap_t* handle, int64_t* pid)
 {
-	if(handle->m_file)
+	if(handle->m_mode != SCAP_MODE_LIVE)
 	{
 		ASSERT(false);
 		return SCAP_FAILURE;
@@ -441,6 +469,7 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 	size_t exe_len;
 	bool free_tinfo = false;
 	int32_t res = SCAP_SUCCESS;
+	struct stat dirstat;
 
 	snprintf(dir_name, sizeof(dir_name), "%s/%u/", procdirname, tid);
 	snprintf(filename, sizeof(filename), "%sexe", dir_name);
@@ -661,12 +690,14 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 		return SCAP_FAILURE;
 	}
 
-	if(scap_get_vtid(handle, tinfo->tid, &tinfo->vtid) == SCAP_FAILURE)
+	// These values should be read already from /status file, leave these
+	// fallback functions for older kernels < 4.1
+	if(tinfo->vtid == 0 && scap_get_vtid(handle, tinfo->tid, &tinfo->vtid) == SCAP_FAILURE)
 	{
 		tinfo->vtid = tinfo->tid;
 	}
 
-	if(scap_get_vpid(handle, tinfo->tid, &tinfo->vpid) == SCAP_FAILURE)
+	if(tinfo->vpid == 0 && scap_get_vpid(handle, tinfo->tid, &tinfo->vpid) == SCAP_FAILURE)
 	{
 		tinfo->vpid = tinfo->pid;
 	}
@@ -681,6 +712,11 @@ static int32_t scap_proc_add_from_proc(scap_t* handle, uint32_t tid, int parentt
 		return SCAP_FAILURE;
 	}
 
+	if(stat(dir_name, &dirstat) == 0)
+	{
+		tinfo->clone_ts = dirstat.st_ctim.tv_sec*1000000000 + dirstat.st_ctim.tv_nsec;
+	}
+
 	//
 	// if tid_to_scan is set we assume this is a runtime lookup so no
 	// need to use the table
@@ -825,11 +861,14 @@ int32_t scap_proc_scan_proc_dir(scap_t* handle, char* procdirname, int parenttid
 		//
 		// See if this process includes tasks that need to be added
 		//
-		snprintf(childdir, sizeof(childdir), "%s/%u/task", procdirname, (int)tid);
-		if(scap_proc_scan_proc_dir(handle, childdir, tid, tid_to_scan, procinfo, error, scan_sockets) == SCAP_FAILURE)
+		if(parenttid == -1 && handle->m_mode != SCAP_MODE_NODRIVER)
 		{
-			res = SCAP_FAILURE;
-			break;
+			snprintf(childdir, sizeof(childdir), "%s/%u/task", procdirname, (int)tid);
+			if(scap_proc_scan_proc_dir(handle, childdir, tid, tid_to_scan, procinfo, error, scan_sockets) == SCAP_FAILURE)
+			{
+				res = SCAP_FAILURE;
+				break;
+			}
 		}
 
 		if(tid_to_scan != -1 && *procinfo)
@@ -895,7 +934,7 @@ struct scap_threadinfo* scap_proc_get(scap_t* handle, int64_t tid, bool scan_soc
 	//
 	// No /proc parsing for offline captures
 	//
-	if(handle->m_file)
+	if(handle->m_mode == SCAP_MODE_CAPTURE)
 	{
 		return NULL;
 	}
@@ -924,7 +963,7 @@ bool scap_is_thread_alive(scap_t* handle, int64_t pid, int64_t tid, const char*
 	//
 	// No /proc parsing for offline captures
 	//
-	if(handle->m_file)
+	if(handle->m_mode == SCAP_MODE_CAPTURE)
 	{
 		return false;
 	}
@@ -967,6 +1006,34 @@ bool scap_is_thread_alive(scap_t* handle, int64_t pid, int64_t tid, const char*
 #endif // HAS_CAPTURE
 }
 
+#if defined(HAS_CAPTURE)
+int scap_proc_scan_proc_table(scap_t *handle)
+{
+	char filename[SCAP_MAX_PATH_SIZE];
+	//
+	// Create the process list
+	//
+	handle->m_lasterr[0] = '\0';
+
+	snprintf(filename, sizeof(filename), "%s/proc", scap_get_host_root());
+	return scap_proc_scan_proc_dir(handle, filename, -1, -1, NULL, handle->m_lasterr, true);
+}
+
+void scap_refresh_proc_table(scap_t* handle)
+{
+	if(handle->m_proclist)
+	{
+		scap_proc_free_table(handle);
+		handle->m_proclist = NULL;
+	}
+	scap_proc_scan_proc_table(handle);
+}
+#else
+void scap_refresh_proc_table(scap_t* handle)
+{
+}
+#endif // HAS_CAPTURE
+
 void scap_proc_free(scap_t* handle, struct scap_threadinfo* proc)
 {
 	scap_fd_free_proc_fd_table(handle, proc);
@@ -988,6 +1055,21 @@ int32_t scap_proc_add(scap_t* handle, uint64_t tid, scap_threadinfo* tinfo)
 	}
 }
 
+int32_t scap_fd_add(scap_threadinfo* tinfo, uint64_t fd, scap_fdinfo* fdinfo)
+{
+	int32_t uth_status = SCAP_SUCCESS;
+
+	HASH_ADD_INT64(tinfo->fdlist, fd, fdinfo);
+	if(uth_status == SCAP_SUCCESS)
+	{
+		return SCAP_SUCCESS;
+	}
+	else
+	{
+		return SCAP_FAILURE;
+	}
+}
+
 //
 // Internal helper functions to output the process table to screen
 //
